##############################################
# R script for RSiena assignment on sex
# segregation in friendship networks. 
# version of 3 February 2017
# written by Christian Steglich
##############################################
# This R-script guides you through an RSiena
# analysis of a dynamic, classroom-based
# friendship network in a Dutch school;
# four waves of data were collected by Andrea
# Knecht in the Dutch school year 2003/04.
# See also file "lab-segregation.pdf".
##############################################
# Contents:
# >>>>>>>>>
#     (1) preparatory steps in R
#     (2) reading data from ASCII or CSV files
#     (3) cursory inspection of the data
#     (4) preparing variables for RSiena
#     (5) model specification / selection
#         of effects (first set of effects)
#     (6) model estimation
#     (7) model modification and re-estimation
##############################################


# (1) preparatory steps in R
# >>>>>>>>>>>>>>>>>>>>>>>>>>>

# if not yet done, install the RSiena package from R-forge:
install.packages("RSiena",repos="http://R-Forge.R-project.org") 

# check installed version:
packageVersion("RSiena")
# should be recent ('1.1.300' or higher)

# load RSiena commands to the R namespace:
library(RSiena)

# check if R working directory contains the data files:
getwd()
list.files()

# if not, set working directory to where the data are:
setwd("<type here the path to the directory where the data are>")
# Windows users need to replace single backslashes by double backslashes! 
list.files()


# (2) reading data from ASCII or CSV files
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

# read the sequence of dependent friendship networks:
friendship.net1 <- as.matrix(read.table("klas03e-friends-waveA.dat",na.strings="9"))
friendship.net2 <- as.matrix(read.table("klas03e-friends-waveB.dat",na.strings="9"))
friendship.net3 <- as.matrix(read.table("klas03e-friends-waveC.dat",na.strings="9"))
friendship.net4 <- as.matrix(read.table("klas03e-friends-waveD.dat",na.strings="9"))
# read dyad-level predictor variable 'primary school friendship':
primary.net <- as.matrix(read.table("klas03e-primary.dat",na.strings="9"))
# read individual-level, demographic predictor variables:
demographics <- read.table("klas03e-demographics.dat",na.strings="0",
	col.names=c('sex','age','ethnicity','religion'))
# In all of these commands, the "na.string" option identifies missing data codes.
# Consult file "klasdata-readme.txt" for data coding details.


# (3) cursory inspection of the data
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

# find out what the number of actors in this data set is:
numberActors <- nrow(demographics)
numberActors

# means of demographic variables:
table(demographics$sex)
table(demographics$age)
table(demographics$ethnicity)
table(demographics$religion)

# take a look at the network change between observations,
# e.g., between the first two:
table(friendship.net1,friendship.net2,useNA='always')
# from such tables, Hamming distances and Jaccard indices can be calculated

# write two functions calculating these measures:
Hamming <- function(changetable) {
	return(changetable[2,1]+changetable[1,2])
}
Jaccard <- function(changetable) {
	return(changetable[2,2]/(changetable[1,2]+changetable[2,1]+changetable[2,2]))
}

# evaluate the functions for the friendship network sequence:
Hamming(table(friendship.net1,friendship.net2))
Hamming(table(friendship.net2,friendship.net3))
Hamming(table(friendship.net3,friendship.net4))
Jaccard(table(friendship.net1,friendship.net2))
Jaccard(table(friendship.net2,friendship.net3))
Jaccard(table(friendship.net3,friendship.net4))
# ... & once more for total change / stability:
Hamming(table(friendship.net1,friendship.net4))
Jaccard(table(friendship.net1,friendship.net4))


# (4) preparing variables for RSiena
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

# identify dependent network variable for the analysis:
friendship <- sienaDependent(array(
	c(friendship.net1,friendship.net2,friendship.net3,friendship.net4),
	dim=c(numberActors,numberActors,4)))
?sienaDependent

# identify actor-level predictor variable for the analysis:
sex.M <- coCovar(demographics[,1],centered=FALSE)
?coCovar
?varCovar

# identify dyad-level predictor variable for the analysis:
primary <- coDyadCovar(primary.net)
?coDyadCovar
?varDyadCovar

# combine the identified data for the analysis:
segregationdata <- sienaDataCreate(friendship,sex.M,primary)
?sienaDataCreate
# how does the object look like when called in R?
segregationdata

# generate an initial descriptive outputfile:
print01Report(segregationdata,
  modelname='segregation-descriptive')
?print01Report
# file "segregation-descriptive.out" was created in the working
# directory, can be inspected for checking the data parsing.


# (5) model specification / selection of effects
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

# create an effects table for model specification:
first.model <- getEffects(segregationdata)
?getEffects

# how does thís 'empty' model specification look like?
first.model
# note that a few effects are pre-selected by default, in
# particular 'reciprocity', which should NOT be part of
# the first model estimated according to step 3 in the
# document "lab-segregation.pdf".

# Which effects are available for adding to the model specification?
effectsDocumentation(first.model)
?effectsDocumentation

# We will now modify the specification according to step 3
# in the document "lab-segregation.pdf";

# include the effect of primary school friendship:
?includeEffects
first.model <- includeEffects(first.model,X,interaction1='primary')
first.model

# include the sex-related sender, receiver, and homophily effects:
first.model <- includeEffects(first.model,egoX,altX,sameX,
	interaction1='sex.M')
first.model

# finally exclude the reciprocity effect:
first.model <- includeEffects(first.model,recip,include=FALSE)
first.model


# (6) model estimation
# >>>>>>>>>>>>>>>>>>>>>

# specify additional options for tuning the estimation algorithm:
segregationoptions <- sienaAlgorithmCreate(useStdInits=FALSE,
	projname='segregation-estimates',cond=FALSE,seed=1234567)
?sienaAlgorithmCreate

# estimate the FIRST model (step 3 in "lab-segregation.pdf"):
?siena07
first.results <- siena07(segregationoptions,data=segregationdata,
	effects=first.model,batch=FALSE,verbose=FALSE)
# Results are written to file "segregation-estimates.out" in the working directory.
# The main ingredients can be seen by calling the results object:
first.results

	# OPTIONAL: If estimation results should show insufficient convergence (i.e., the
	# convergence t-ratios are not all between -0.1 and +0.1 for the estimated(!)
	# parameters), another run with better starting values may help.
	# The option 'prevAns' makes it possible to use the last available estimates
	# as starting values for a new run:
	first.results <- siena07(segregationoptions,data=segregationdata,
		effects=first.model,batch=FALSE,verbose=FALSE,prevAns=first.results)
	first.results
	# Convergence statistics should have improved and be okay before proceeding.
	# Note that new results will also be appended to the existing file
	# "segregation-estimates.out", so scroll down there for the latest estimates.


# (7) model modification and re-estimation
# >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>

# for the SECOND model (step 4 in "lab-segregation.pdf"), we include the
# effects of reciprocity, transitivity, and their interaction:
second.model <- includeEffects(first.model,recip,transTrip,transRecTrip)
second.model

# estimate SECOND model:
second.results <- siena07(segregationoptions,data=segregationdata,
	effects=second.model,batch=FALSE,verbose=FALSE)
# Results are appended to file "segregation-estimates.out" in the working directory.
second.results

# If you look at the two results, you see the parameter estimate of 'same sex.M'
# drop when the the network-structural parameters are included. When comparing
# the first and the second model, there even is a significant difference in model
# results: 95% confidence intervals for this parameter, which can be constructed
# with the help of the standard error estimates, do not overlap!

# save results if needed later:
save.image("segregation lab results.RData")
# this can later be loaded again
